//
//  UIAdapter.swift
//  TestUI
//
//  Created by Cortes Saenz, David on 2/9/16.
//  Copyright © 2016 Akamai. All rights reserved.
//

import Foundation
import AmpCore
import AmpUICore
import AVFoundation

public class UIAdapter: NSObject, PlayerEventObserver, UIEventObserver {
    let bottomBar:BottomBar
    let topBar:TopBar
    let amplayerManager:AmpPlayerManager
    let ampPlayer: AmpPlayer
    var buttonPlay:Button?
    var seekBar:SeekBar?
    var labelTime:Label?
    var buttonFullScreen:Button?
    var buttonFullSettings:Button?
    var popOver:PopOverMenu?
    var parent:UIViewController?
    let spinner:UIActivityIndicatorView
    var isFullScreen = false
    
    
    
    public init(amplayerManager: AmpPlayerManager, parentViewController:UIViewController){
        self.amplayerManager = amplayerManager
        self.ampPlayer = self.amplayerManager.ampPlayer
        self.ampPlayer.allowAirplayPlayback(true)
        
        spinner = UIActivityIndicatorView(activityIndicatorStyle: .WhiteLarge)
        let spinnerSize: CGFloat = 50.0;
        let frame = self.amplayerManager.playerView.frame
        spinner.frame =  CGRect(x: (frame.width / 2) - (spinnerSize / 2) , y: (frame.height / 2) - (spinnerSize / 2), width: spinnerSize, height: spinnerSize)
        spinner.startAnimating()
        
        self.amplayerManager.playerView.addSubview(spinner)
        
        bottomBar = BottomBar(superview: self.amplayerManager.playerView, height: 40)
        bottomBar.setAutoHide(4.0)
        topBar = TopBar(superview: self.amplayerManager.playerView, height: 30)
        topBar.setAutoHide(4.0)
        
        if let background = UIImage(named: "background") {
            bottomBar.setBackgroundImage(background)
            topBar.setBackgroundImage(background)
        }
        
        let button1 = UIButton(type: .Custom)
        if let image = UIImage(named: "play") {
            button1.setImage(image, forState: .Normal)
            button1.imageView?.contentMode = .ScaleAspectFill
        }
        
        super.init()
        
        self.buttonPlay = Button(view: button1, action: { () in
            if self.ampPlayer.playbackState == .Playing {
                self.ampPlayer.pause()
            }else {
                self.ampPlayer.play()
            }
        })
        bottomBar.addElementToView(buttonPlay!, size: 2)
        
        let slider = UISlider()
        
        self.seekBar = SeekBar(view: slider, observer: self, player: self.ampPlayer)
        
        if let sliderImage = UIImage(named: "slider") {
            seekBar!.setMaximumTrackImage(sliderImage, controlState: .Normal)
            
            
        }
        
        if let sliderProgress = UIImage(named: "progress") {
            seekBar!.setMinimumTrackImage(sliderProgress, controlState: .Normal)
        }
        
        if let sliderThumb = UIImage(named: "silder_thumb") {
            seekBar!.setThumbImage(sliderThumb, controlState: .Normal)
        }
        
        
        bottomBar.addElementToView(seekBar!, size: 6)
        
        let label = UILabel()
        label.text = "00:00:00"
        label.textColor = UIColor.whiteColor()
        label.font = UIFont(name: "HelveticaNeue", size: 10)
        label.textAlignment = .Center
        
        self.labelTime = Label(view: label, observer: self)
        
        bottomBar.addElementToView(labelTime!, size: 2)
        
        let button3 = UIButton(type: .Custom)
        
        if let imageFull = UIImage(named: "settings") {
            button3.setImage(imageFull, forState: .Normal)
            button3.imageView?.contentMode = .ScaleAspectFill
        }
        
        let button2 = UIButton(type: .Custom)
        
        if let imageFull = UIImage(named: "full") {
            button2.setImage(imageFull, forState: .Normal)
            button2.imageView?.contentMode = .ScaleAspectFill
        }

        self.buttonFullScreen = Button(view: button2, action: { () in
            
            if !self.isFullScreen {
                self.amplayerManager.enterFullscreen()
            }else {
                self.amplayerManager.exitFullscreen()
            }
        })
        
        
        bottomBar.addElementToView(buttonFullScreen!, size: 2)
        
        self.popOver = PopOverMenu(frame: CGRect(x: 0, y: 0, width: 100, height: 120), buttons: [], observer: self)
        
        self.parent = parentViewController
        
        self.ampPlayer.registerObserver(self)
    }
    
    // MARK: PlayerEventObserver
    
    public func onBufferingStateChanged(ampPlayer: AmpPlayer) {
        if (ampPlayer.bufferingState == .Ready) {
            // Now that the player is ready to be played...
            
            for audio in ampPlayer.getAvailableCloseCaptions() {
                self.popOver?.buttons!.append(PopOverButton(image: UIImage(), text: audio))
            }
            
            if (ampPlayer.isLive()) {
                // If it's a live stream
                
                // set it to be always at it's maximum value
                self.seekBar!.maximumValue = 1.0
                self.seekBar!.currentValue = 1.0
                // disable the seekbar
                self.seekBar!.disable()
            } else {
                // otherwise set the stream's duration
                self.seekBar!.maximumValue = Float(ampPlayer.duration())
            }
            
            // enable the playPauseButton
            self.buttonPlay!.enable()
            
            // hide the spinner
            self.spinner.hidden = true
        } else {
            // Since the player is hasn't finished loading...
            
            // show the spinner
            self.spinner.hidden = false
            
            // disable the playPauseButton
            self.buttonPlay!.disable()
        }
    }
    
    public func onTimeMetadata(ampPlayer: AmpPlayer, metadata: [AVMetadataItem]) {
        for item in metadata {
            print(item.value!)
        }
    }
    
    public func onTap(ampPlayer: AmpPlayer) {
        self.bottomBar.show()
        self.topBar.show()
        self.bottomBar.resetAutoHide()
        self.topBar.resetAutoHide()
        
    }
    
    public func onPlaybackStateChanged(ampPlayer: AmpPlayer) {
        switch ampPlayer.playbackState {
            
        case PlaybackState.Playing:
            // If the player is playing we show the pause button
            if let image = UIImage(named: "pause") {
                self.buttonPlay!.setImage(image, controlState: .Normal)
            }
        case PlaybackState.Paused:
            // If the player is paused we show the play button
            if let image = UIImage(named: "play") {
                self.buttonPlay!.setImage(image, controlState: .Normal)
            }
        default:
            break
        }
    }
    
    // MARK: UI Event Observer
    
    public func onPositionChange(ampPlayer: AmpPlayer) {
        // Grab the current time from the player
        let currentTime = Float(ampPlayer.currentTime())
        // Update the seekbar's position if the stream is not live
        if (!ampPlayer.isLive()) {
            self.seekBar!.sync(currentTime)
        }
        // Update the label's time
        self.labelTime!.updateLabelWithTime(currentTime)
    }
    
    public func onSeekBegan(seekbar: SeekBar) {
        self.ampPlayer.pause()
    }
    
    public func onSeekFinished(seekbar: SeekBar) {
        self.ampPlayer.seekTo(Double(seekbar.currentValue))
        self.ampPlayer.play()
    }
    
    public func onValueChanged(seekbar: SeekBar) {
        self.labelTime?.updateLabelWithTime(seekbar.currentValue)
    }
    
    public func onPopOverButtonPress(index: Int) {
        let option = self.ampPlayer.getAvailableCloseCaptions()[index]
        self.ampPlayer.setMediaCharacteristic(option, type: .CloseCaptions)
    }
    
    public func willEnterFullscreen(ampPlayer: AmpPlayer, newFrame: CGRect) {
        self.isFullScreen = true
        self.bottomBar.updateBarFrameWithFrame(newFrame)
        self.topBar.updateBarFrameWithFrame(newFrame)
    }
    
    public func willExitFullscreen(ampPlayer: AmpPlayer, newFrame: CGRect) {
        self.isFullScreen = false
        self.bottomBar.updateBarFrameWithFrame(newFrame)
        self.topBar.updateBarFrameWithFrame(newFrame)
    }
    
    public func onPlayerRotate(ampPlayerManager: AmpPlayerManager, orientation: UIInterfaceOrientation, newFrame: CGRect) {
        self.bottomBar.updateBarFrameWithFrame(newFrame)
        self.topBar.updateBarFrameWithFrame(newFrame)
        if (orientation != .LandscapeLeft || orientation != .LandscapeRight) && self.isFullScreen == true {
            self.amplayerManager.enterFullscreen()
        }else {
            self.amplayerManager.exitFullscreen()
        }
    }
    
    
}
